@@ -9,7 +9,10 @@
 // XXX: allocate this dynamically?
 #define MAXCSLEN 64
 
-typedef enum {
+namespace acng
+{
+
+typedef enum : char {
    CSTYPE_INVALID=0,
    CSTYPE_MD5=1,
    CSTYPE_SHA1=2,
@@ -271,5 +274,6 @@ struct ltCacheKeyComp
 
 typedef std::map<tImpFileInfo, tFingerprint, ltCacheKeyComp> tFprCacheMap;
 
+}
 
 #endif /*CSMAPPING_H_*/
