@@ -1076,7 +1076,7 @@ void PostProcConfig()
 		port=ACNG_DEF_PORT;
 
 	if(connectPermPattern == "~~~")
-	   connectPermPattern="^bugs\\.debian\\.org:443$";
+	   connectPermPattern="^(bugs\\.debian\\.org|changelogs\\.ubuntu\\.com):443$";
 
 	// let's also apply the umask to the directory permissions
 	{
