@@ -22,6 +22,12 @@ using namespace std;
 #undef SendFmtRemote
 #endif
 
+#define SCALEFAC 250
+
+
+namespace acng
+{
+
 static cmstring sReportButton("<tr><td class=\"colcont\"><form action=\"#stats\" method=\"get\">"
 					"<input type=\"submit\" name=\"doCount\" value=\"Count Data\"></form>"
 					"</td><td class=\"colcont\" colspan=8 valign=top><font size=-2>"
@@ -48,8 +54,8 @@ void tMarkupFileSend::Run()
 	const char *pr(nullptr), *pend(nullptr);
 	if(!m_bFatalError)
 	{
-		m_bFatalError = ! ( fr.OpenFile(acfg::confdir+SZPATHSEP+m_sFileName, true) ||
-			(!acfg::suppdir.empty() && fr.OpenFile(acfg::suppdir+SZPATHSEP+m_sFileName, true)));
+		m_bFatalError = ! ( fr.OpenFile(cfg::confdir+SZPATHSEP+m_sFileName, true) ||
+			(!cfg::suppdir.empty() && fr.OpenFile(cfg::suppdir+SZPATHSEP+m_sFileName, true)));
 	}
 	if(m_bFatalError)
 	{
@@ -131,6 +137,8 @@ tDeleter::tDeleter(const tRunParms& parms, const mstring& vmode)
 		return;
 	}
 
+	auto del = (m_parms.type == workDELETE);
+
 	mstring params(m_parms.cmd, qpos+1);
 	mstring blob;
 	for(tSplitWalk split(&params, "&"); split.Next();)
@@ -139,7 +147,7 @@ tDeleter::tDeleter(const tRunParms& parms, const mstring& vmode)
 		if(startsWithSz(tok, "kf="))
 		{
 			char *end(0);
-			auto val = strtoul(tok.c_str()+3, &end, 16);
+			auto val = strtoul(tok.c_str()+3, &end, 36);
 			if(*end == 0 || *end=='&')
 #ifdef COMPATGCC47                           
 				files.insert(val);
@@ -151,11 +159,14 @@ tDeleter::tDeleter(const tRunParms& parms, const mstring& vmode)
 			tok.swap(blob);
 	}
 	sHidParms << "<input type=\"hidden\" name=\"blob\" value=\"";
-	sHidParms.append(blob.data()+5, blob.size()-5);
+	if(!blob.empty())
+		sHidParms.append(blob.data()+5, blob.size()-5);
 	sHidParms <<  "\">\n";
 
 	tStrDeq filePaths;
 	acbuf buf;
+	mstring redoLink;
+
 #ifdef HAVE_DECB64 // this page isn't accessible with crippled configuration anyway
 	if (!blob.empty())
 	{
@@ -189,7 +200,9 @@ tDeleter::tDeleter(const tRunParms& parms, const mstring& vmode)
 		buf.drop(sizeof(unsigned));
 		if(slen > buf.size()) // looks fishy
 			return;
-		if(ContHas(files, id))
+		if(redoLink.empty()) // don't care about id in the first line
+			redoLink.assign(buf.rptr(), slen);
+		else if(ContHas(files, id))
 			filePaths.emplace_back(buf.rptr(), slen);
 		buf.drop(slen);
 	}
@@ -198,39 +211,81 @@ tDeleter::tDeleter(const tRunParms& parms, const mstring& vmode)
 	for(const auto& path : filePaths)
 	{
 		if(path.find_first_of(BADCHARS)!=stmiss  // what the f..., XSS attempt?
-		 || rechecks::Match(path, rechecks::NASTY_PATH))
+		 || rex::Match(path, rex::NASTY_PATH))
 		{
 			m_bFatalError=true;
 			return;
 		}
 	}
 
+	// XXX: this is wasting some CPU cycles but is good enough for this case
+	for (const auto& path : filePaths)
+	{
+		mstring bname(path);
+		for(const auto& sfx: sfxXzBz2GzLzma)
+			if(endsWith(path, sfx))
+				bname = path.substr(0, path.size()-sfx.size());
+		auto tryAdd=[this,&bname,&path](cmstring& sfx)
+				{
+					auto cand = bname+sfx;
+					if(cand == path || ::access(SZABSPATH(cand), F_OK))
+						return;
+					extraFiles.push_back(cand);
+				};
+		for(const auto& sfx: sfxMiscRelated)
+			tryAdd(sfx);
+		if(endsWith(path, relKey))
+			tryAdd(path.substr(0, path.size()-relKey.size())+inRelKey);
+		if(endsWith(path, inRelKey))
+			tryAdd(path.substr(0, path.size()-inRelKey.size())+relKey);
+	}
+
+
 	if (m_parms.type == workDELETECONFIRM || m_parms.type == workTRUNCATECONFIRM)
 	{
 		for (const auto& path : filePaths)
 			sHidParms << html_sanitize(path) << "<br>\n";
 		for (const auto& pathId : files)
-			sHidParms << "<input type=\"hidden\" name=\"kf\" value=\"" << tSS::fmtflags::hex
-					<< pathId << "\">\n" << tSS::fmtflags::dec;
+			sHidParms << "<input type=\"hidden\" name=\"kf\" value=\"" <<
+			to_base36(pathId) << "\">\n";
+		if(m_parms.type == workDELETECONFIRM && !extraFiles.empty())
+		{
+			sHidParms << sBRLF << "<b>Extra files found</b>" << sBRLF
+					<< "<p>It's recommended to delete the related files (see below) as well, otherwise "
+					<< "the removed files might be resurrected by recovery mechanisms later.<p>"
+					<< "<input type=\"checkbox\" name=\"cleanRelated\" value=\"1\" checked=\"checked\">"
+					<< "Yes, please remove all related files<p>Example list:<p>";
+			for (const auto& path : extraFiles)
+				sHidParms << path << sBRLF;
+		}
 	}
 	else
 	{
-		auto del = (m_parms.type == workDELETE);
 		for (const auto& path : filePaths)
 		{
-			for (auto suf :
-			{ "", ".head" })
-			{
-				sHidParms << (del ? "Deleting " : "Truncating ") << path << suf << "<br>\n";
-				auto p = acfg::cacheDirSlash + path + suf;
-				int r = del ? unlink(p.c_str()) : truncate(p.c_str(), 0);
-				if (r && errno != ENOENT)
+			auto doFile=[this, &del](cmstring& path)
+					{
+				for (auto suf : { "", ".head" })
 				{
-					tErrnoFmter ferrno("<span class=\"ERROR\">[ error: ");
-					sHidParms << ferrno << " ]</span><br>\n";
+					sHidParms << (del ? "Deleting " : "Truncating ") << path << suf << "<br>\n";
+					auto p = cfg::cacheDirSlash + path + suf;
+					int r = del ? unlink(p.c_str()) : truncate(p.c_str(), 0);
+					if (r && errno != ENOENT)
+					{
+						tErrnoFmter ferrno("<span class=\"ERROR\">[ error: ");
+						sHidParms << ferrno << " ]</span>" << sBRLF;
+					}
+					if(!del)
+						break;
 				}
-			}
+			};
+			doFile(path);
+			if(StrHas(m_parms.cmd, "cleanRelated="))
+				for (const auto& path : extraFiles)
+					doFile(path);
+
 		}
+		sHidParms << "<br><a href=\""<< redoLink << "\">Repeat the last action</a><br>" << sBRLF;
 	}
 }
 
@@ -239,9 +294,9 @@ tMaintPage::tMaintPage(const tRunParms& parms)
 {
 
 	if(StrHas(parms.cmd, "doTraceStart"))
-		acfg::patrace=true;
+		cfg::patrace=true;
 	else if(StrHas(parms.cmd, "doTraceStop"))
-		acfg::patrace=false;
+		cfg::patrace=false;
 	else if(StrHas(parms.cmd, "doTraceClear"))
 	{
 		auto& tr(tTraceData::getInstance());
@@ -260,12 +315,12 @@ inline int tMarkupFileSend::CheckCondition(LPCSTR id, size_t len)
 	if(PFXCMP(id, len, "cfg:"))
 	{
 		string key(id+4, len-4);
-		auto p=acfg::GetIntPtr(key.c_str());
+		auto p=cfg::GetIntPtr(key.c_str());
 		if(p)
 			return ! *p;
-    if(key == "degraded")
-       return acfg::degraded.load();
-		return -1;
+		if(key == "degraded")
+			return cfg::DegradedMode();
+    	return -1;
 	}
 	if(RAWEQ(id, len, "delConfirmed"))
 		return m_parms.type != workDELETE && m_parms.type != workTRUNCATE;
@@ -303,11 +358,11 @@ void tMaintPage::SendProp(cmstring &key)
 	{
 		if(!StrHas(m_parms.cmd, "doCount"))
 			return SendChunk(sReportButton);
-		return SendChunk(aclog::GetStatReport());
+		return SendChunk(log::GetStatReport());
 	}
 	static cmstring defStringChecked("checked");
 	if(key == "aOeDefaultChecked")
-		return SendChunk(acfg::exfailabort ? defStringChecked : sEmptyString);
+		return SendChunk(cfg::exfailabort ? defStringChecked : sEmptyString);
 	if(key == "curPatTraceCol")
 	{
 		m_fmtHelper.clear();
@@ -349,10 +404,10 @@ void tMarkupFileSend::SendProp(cmstring &key)
 	if (startsWithSz(key, "cfg:"))
 	{
 		auto ckey=key.c_str() + 4;
-		auto ps(acfg::GetStringPtr(ckey));
+		auto ps(cfg::GetStringPtr(ckey));
 		if(ps)
 			return SendChunk(*ps);
-		auto pi(acfg::GetIntPtr(ckey));
+		auto pi(cfg::GetIntPtr(ckey));
 		if(pi)
 			return SendChunk(m_fmtHelper.clean() << *pi);
 		return;
@@ -371,4 +426,57 @@ void tMarkupFileSend::SendProp(cmstring &key)
 	}
 	if(key=="random")
 		return SendChunk(m_fmtHelper.clean() << rand());
+	if(key=="dataInHuman")
+	{
+		auto stats = log::GetCurrentCountersInOut();
+		return SendChunk(offttosH(stats.first));
+	}
+	if(key=="dataOutHuman")
+	{
+		auto stats = log::GetCurrentCountersInOut();
+		return SendChunk(offttosH(stats.second));
+	}
+	if(key=="dataIn")
+	{
+		auto stats = log::GetCurrentCountersInOut();
+		auto statsMax = std::max(stats.first, stats.second);
+		auto pixels = statsMax ? (stats.first * SCALEFAC / statsMax) : 0;
+		return SendChunk(m_fmtHelper.clean() << pixels);
+	}
+	if(key=="dataOut")
+	{
+		auto stats = log::GetCurrentCountersInOut();
+		auto statsMax = std::max(stats.second, stats.first);
+		auto pixels = statsMax ? (SCALEFAC * stats.second / statsMax) : 0;
+		return SendChunk(m_fmtHelper.clean() << pixels);
+	}
+
+	if (key == "dataHistInHuman")
+	{
+		auto stats = pairSum(log::GetCurrentCountersInOut(), log::GetOldCountersInOut());
+		return SendChunk(offttosH(stats.first));
+	}
+	if (key == "dataHistOutHuman")
+	{
+		auto stats = pairSum(log::GetCurrentCountersInOut(), log::GetOldCountersInOut());
+		return SendChunk(offttosH(stats.second));
+	}
+	if (key == "dataHistIn")
+	{
+		auto stats = pairSum(log::GetCurrentCountersInOut(), log::GetOldCountersInOut());
+		auto statsMax = std::max(stats.second, stats.first);
+		auto pixels = statsMax ? (stats.first * SCALEFAC / statsMax) : 0;
+		return SendChunk(m_fmtHelper.clean() << pixels);
+	}
+	if (key == "dataHistOut")
+	{
+		auto stats = pairSum(log::GetCurrentCountersInOut(), log::GetOldCountersInOut());
+		auto statsMax = std::max(stats.second, stats.first);
+		auto pixels = statsMax ? (SCALEFAC * stats.second/statsMax) : 0;
+		return SendChunk(m_fmtHelper.clean() << pixels);
+	}
+
+
+}
+
 }
